using System;
using System.Runtime.InteropServices;

namespace mp2enc
{
	/// <summary>
	/// libTwoLAME Wrapper Class
	///   descriptions are imported from "twolame.h"
	/// </summary>
	public unsafe class Twolame
	{
		/// dll name
		const string dllname = "libtwolame.dll";
		private void* options;

		/// <summary>
		///  MPEG mode
		/// </summary>
		public enum MPEG_mode
		{
			AUTO_MODE = -1,	/// Auto select
			STEREO = 0,		/// Stereo
			JOINT_STEREO,	/// Joint Stereo
			DUAL_CHANNEL,	/// Dual Channel
			MONO,			/// Mono
			NOT_SET
		}

		/// <summary>
		/// MPEG Version
		/// </summary>
		public enum MPEG_version
		{
			MPEG2 = 0,	/// MPEG-2  16k, 22.05k, 24k
			MPEG1		/// MPEG-1  32k, 44.1k,  48k
		}

		/// <summary>
		/// Padding type
		/// </summary>
		public enum Padding
		{
			PAD_NO = 0,		/// No Padding
			PAD_ALL			/// Pad all frames
			//PAD_ADJUST
		}

		/// <summary>
		/// Emphasis types
		/// </summary>
		public enum Emphasis
		{
			EMPHASIS_N = 0,	/// No Emphasis
			EMPHASIS_5 = 1,	/// 50/15 ms
			// reserved
			EMPHASIS_C = 3,	/// CCIT J.17 
		}


		// Opaque structure for the twolame encoder options.
		//struct twolame_options_struct;

		// Opaque data type for the twolame encoder options.
		//typedef struct twolame_options_struct twolame_options;

		/// <summary>
		/// Constructor
		/// </summary>
		public Twolame()
		{
			//init
			options = null;
		}

		~Twolame()
		{
			fixed(void** p = &options)
			{
				if(*p != null)
				{
					twolame_close(p);
				}
			}
		}

		/// <summary>
		/// Get the version number of the TwoLAME library. 
		/// </summary>
		/// <returns>version number string. (eg "0.3.1")</returns>
		public static string get_version()
		{
			return new string(get_twolame_version());
		}
		[DllImport(dllname)]
		private static extern sbyte* get_twolame_version();
		
		/// <summary>
		/// Get the URL of the TwoLAME homepage.
		/// </summary>
		/// <returns>The url string. (eg "http://www.twolame.org/")</returns>
		///
		public static string get_url()
		{
			return new string(get_twolame_url());
		}
		[DllImport(dllname)]
		private static extern sbyte* get_twolame_url();



		/// <summary>
		/// Print the library version and 
		/// encoder parameter settings to STDERR.
		/// </summary>
		/// <param name="options">Options pointer created by twolame_init()</param>
		[DllImport(dllname)]
		private static extern void twolame_print_config(void *options);


		/// <summary>
		/// Initialise the twolame encoder.
		///  Sets defaults for all parameters.
		/// </summary>
		/// <returns>false on failure</returns>
		public bool init()
		{
			options = twolame_init();
			if(options != null)
			{
				set_verbosity(0);
				return true;
			}

			return false;
		}
		[DllImport(dllname)]
		private static extern void * twolame_init();


		/// <summary>
		/// Prepare to start encoding.
		///  You must call before you start encoding.
		/// </summary>
		/// <returns>0 if all patameters are valid, non-zero if something is invalid.</returns>
		public int init_params()
		{
			return twolame_init_params(options);
		}
		[DllImport(dllname)]
		private static extern int twolame_init_params(void *options);


		/// <summary>
		/// Encode PCM audio to MP2.
		///  Takes 16-bit PCM audio samples from seperate left and right
		/// </summary>
		/// <param name="lp">Left channel audio samples</param>
		/// <param name="rp">Right channel audio samples</param>
		/// <param name="num_samples">Number of samples per channel</param>
		/// <param name="mp2buf">output buffer</param>
		/// <param name="sz_mp2buf">Size of the output buffer</param>
		/// <returns>number of bytes put in output buffer. negative value on error</returns>
		public int encode_buffer(byte[] lp, byte[] rp, int num_samples, byte[] mp2buf, int sz_mp2buf)
		{
			return twolame_encode_buffer(options, lp, rp, num_samples, mp2buf, sz_mp2buf);
		}
		[DllImport(dllname)]
		private static extern int twolame_encode_buffer(
			void* options,
			byte[] leftpcm,
			byte[] rightpcm,
			int num_samples,
			byte[] mp2buffer,
			int mp2buffer_size );


		/// <summary>
		/// Encode PCM audio to MP2.
		///  Takes interleaved 16-bit PCM audio samples
		/// </summary>
		/// <param name="pcm">Audio samples</param>
		/// <param name="num_samples">Number of samples per channel</param>
		/// <param name="mp2buffer">output buffer</param>
		/// <param name="mp2buffer_size">Size of the output buffer</param>
		/// <returns>number of bytes put in output buffer. negative value on error</returns>
		public int encode_buffer_interleaved(byte[] pcm, int num_samples, byte[] mp2buf, int sz_mp2buf)
		{
			return twolame_encode_buffer_interleaved(options, pcm, num_samples, mp2buf, sz_mp2buf);
		}
		[DllImport(dllname)]
		private static extern int twolame_encode_buffer_interleaved(
			void* options,
			byte[] pcm,
			int num_samples,
			byte[] mp2buffer,
			int mp2buffer_size );


		/// <summary>
		/// Encode any remains buffered PCM audio to MP2.
		///  Encodes any remaining audio samples in the internal sample buffer.
		/// </summary>
		/// <param name="buf">output buffer</param>
		/// <param name="sz_buf">Size of the output buffer</param>
		/// <returns>number of bytes put in output buffer. negative value on error</returns>
		public int encode_flush(byte[] buf, int sz_buf)
		{
			return twolame_encode_flush(options, buf, sz_buf);
		}
		[DllImport(dllname)]
		private static extern int twolame_encode_flush(
			void* options,
			byte[] mp2buffer,
			int mp2buffer_size);


		/// <summary>
		/// Shut down the twolame encoder.
		/// </summary>
		public void close()
		{
			fixed(void** p = &options)
			{
				if(*p != null)
				{
					twolame_close(p);
				}
			}
		}
		[DllImport(dllname)]
		private static extern void twolame_close(void **options);
		

		/// <summary>
		/// Set the verbosity of the encoder.
		/// </summary>
		/// <param name="verbosity">integer between 0 and 10.(Default:1)</param></param>
		/// <returns>0 if successful</returns>
		public int set_verbosity(int verbosity)
		{
			return twolame_set_verbosity(options, verbosity);
		}
		[DllImport(dllname)]
		private static extern int twolame_set_verbosity(void *options, int verbosity);


		/// <summary>
		/// Get the verbosity of the encoder.
		/// </summary>
		/// <returns>nteger indicating the verbosity of the encoder (0-10)</returns>
		public int get_verbosity()
		{
			return twolame_get_verbosity(options);
		}
		[DllImport(dllname)]
		private static extern int twolame_get_verbosity(void *options);


		/// <summary>
		/// Set the MPEG Audio Mode (Mono, Stereo, etc) for the output stream.
		/// </summary>
		/// <param name="mode">the mode to set to(Default:AUTO_MODE)</param>
		/// <returns>0 if successful</returns>
		public int set_mode(MPEG_mode mode)
		{
			return twolame_set_mode(options, mode);
		}
		[DllImport(dllname)]
		private static extern int twolame_set_mode(void *options, MPEG_mode mode);


		/// <summary>
		/// Get the MPEG Audio mode of the output stream.
		/// </summary>
		/// <returns>MPEG audio mode</returns>
		public MPEG_mode get_mode()
		{
			return twolame_get_mode(options);
		}
		[DllImport(dllname)]
		private static extern MPEG_mode twolame_get_mode(void *options);


		/// <summary>
		/// Get a string name for the current MPEG Audio mode.
		/// </summary>
		/// <returns>MPEG audio mode as a string</returns>
		public string get_mode_name()
		{
			return new string(twolame_get_mode_name(options));
		}
		[DllImport(dllname)]
		private static extern sbyte* twolame_get_mode_name(void *options);


		/// <summary>
		/// Set the MPEG version of the MPEG audio stream. 
		/// </summary>
		/// <param name="version">the version to set to(Default:MPEG1)</param>
		/// <returns></returns>
		public int set_mpeg_version(MPEG_version version)
		{
			return twolame_set_version(options, version);
		}
		[DllImport(dllname)]
		private static extern int twolame_set_version(void *options, MPEG_version version);


		/// <summary>
		/// Get the MPEG version of the output stream.
		/// </summary>
		/// <returns>MPEG version</returns>
		public MPEG_version get_mpeg_version()
		{
			return twolame_get_version(options);
		}
		[DllImport(dllname)]
		private static extern MPEG_version twolame_get_version( void *options);

	
		/// <summary>
		/// Get a string name for the current MPEG version.
		/// </summary>
		/// <returns>MPEG version as a string</returns>
		public string get_mpeg_version_name()
		{
			return new string(twolame_get_version_name(options));
		}
		[DllImport(dllname)]
		private static extern sbyte* twolame_get_version_name( void *options );


		/// <summary>
		/// Set the Psychoacoustic Model used to encode the audio.
		/// </summary>
		/// <param name="psymodel">psychoacoustic model number -1 to 4(Default:3)</param>
		/// <returns></returns>
		public int set_psymodel(int psymodel)
		{
			return twolame_set_psymodel(options, psymodel);
		}
		[DllImport(dllname)]
		private static extern int twolame_set_psymodel(void *options, int psymodel);


		/// <summary>
		/// Get the Psychoacoustic Model used to encode the audio.
		/// </summary>
		/// <returns>the psychoacoustic model number</returns>
		public int get_psymodel()
		{
			return twolame_get_psymodel(options);
		}
		[DllImport(dllname)]
		private static extern int twolame_get_psymodel(void *options);


		/// <summary>
		/// Set the number of channels in the input stream.
		///  If this is different the number of channels in
		///  the output stream (set by mode) then the encoder
		///  will automatically downmix/upmix the audio.
		/// </summary>
		/// <param name="num_channels">the number of input channels(Default:2)</param>
		/// <returns>0 if successful</returns>
		public int set_num_channels(int num_channels)
		{
			return twolame_set_num_channels(options, num_channels);
		}
		[DllImport(dllname)]
		private static extern int twolame_set_num_channels(void* options, int num_channels);


		/// <summary>
		/// Get the number of channels in the input stream.
		/// </summary>
		/// <returns>the number of channels</returns>
		public int get_num_channels()
		{
			return twolame_get_num_channels(options);
		}
		[DllImport(dllname)]
		private static extern int twolame_get_num_channels(void* options);


		/// <summary>
		/// Set the scaling level for audio before encoding.
		///  Set to 0 to disable.
		/// </summary>
		/// <param name="scale">scaler(Default: 0 (means 1.0))</param>
		/// <returns>0 if successful</returns>
		public int set_scale(float scale)
		{
			return twolame_set_scale(options, scale);
		}
		[DllImport(dllname)]
		private static extern int twolame_set_scale(void* options, float scale);


		/// <summary>
		/// Get the scaling level for audio before encoding.
		/// </summary>
		/// <returns>the amount to scale audio sample by</returns>
		public float get_scale()
		{
			return twolame_get_scale(options);
		}
		[DllImport(dllname)]
		private static extern float twolame_get_scale(void* options);
 

		/// <summary>
		/// Set the scaling level for left channel audio before encoding.
		/// </summary>
		/// <param name="scale">scaler(Default: 0 (means 1.0))</param>
		/// <returns>0 if successful</returns>
		public float set_scale_left(float scale)
		{
			return twolame_set_scale_left(options, scale);
		}
		[DllImport(dllname)]
		private static extern int twolame_set_scale_left(void* options, float scale);


		/// <summary>
		/// Get the scaling level for audio left channel before encoding.
		/// </summary>
		/// <returns>the amount to scale left channel audio samples by</returns>
		public float get_scale_left()
		{
			return twolame_get_scale_left(options);
		}
		[DllImport(dllname)]
		private static extern float twolame_get_scale_left(void* options);


		/// <summary>
		/// Set the scaling level for right channel audio before encoding.
		/// </summary>
		/// <param name="scale">scaler(Default: 0 (means 1.0))</param>
		/// <returns>0 if successful</returns>
		public float set_scale_right(float scale)
		{
			return twolame_set_scale_right(options, scale);
		}
		[DllImport(dllname)]
		private static extern int twolame_set_scale_right(void* options, float scale);


		/// <summary>
		/// Get the scaling level for audio right channel before encoding.
		/// </summary>
		/// <returns>the amount to scale right channel audio samples by</returns>
		public float get_scale_right()
		{
			return twolame_get_scale_right(options);
		}
		[DllImport(dllname)]
		private static extern float twolame_get_scale_right(void* options);


		/// <summary>
		/// Set the samplerate of the PCM audio input.
		/// </summary>
		/// <param name="samplerate">samplerate in Hz(Default:44100)</param>
		/// <returns>0 if successful</returns>
		public int set_in_samplerate(int samplerate)
		{
			return twolame_set_in_samplerate(options, samplerate);
		}
		[DllImport(dllname)]
		private static extern int twolame_set_in_samplerate(void* options, int samplerate);
		

		/// <summary>
		/// Get the samplerate of the PCM audio input.
		/// </summary>
		/// <returns>the input samplerate</returns>
		public int get_in_samplerate()
		{
			return twolame_get_in_samplerate(options);
		}
		[DllImport(dllname)]
		private static extern int twolame_get_in_samplerate(void* options);

		
		/// <summary>
		/// Set the samplerate of the MPEG audio output.
		/// </summary>
		/// <param name="samplerate">samplerate in Hz(Default:44100)</param>
		/// <returns>0 if successful</returns>
		public int set_out_samplerate(int samplerate)
		{
			return twolame_set_out_samplerate(options, samplerate);
		}
		[DllImport(dllname)]
		private static extern int twolame_set_out_samplerate(void* options, int samplerate);


		/// <summary>
		/// Get the samplerate of the MPEG audio output.
		/// </summary>
		/// <returns>output samplerate</returns>
		public int get_out_samplerate()
		{
			return twolame_get_out_samplerate(options);
		}
		[DllImport(dllname)]
		private static extern int twolame_get_out_samplerate(void *options);


		/// <summary>
		/// Set the bitrate of the MPEG audio output stream.
		/// </summary>
		/// <param name="bitrate">bitrate in kbps(Default:192)</param>
		/// <returns>0 if successful</returns>
		public int set_bitrate(int bitrate)
		{
			return twolame_set_bitrate(options, bitrate);
		}
		[DllImport(dllname)]
		private static extern int twolame_set_bitrate(void *options, int bitrate);


		/// <summary>
		/// Get the bitrate of the MPEG audio output.
		/// </summary>
		/// <returns>output bitrate in kbps</returns>
		public int get_bitrate()
		{
			return twolame_get_bitrate(options);
		}
		[DllImport(dllname)]
		private static extern int twolame_get_bitrate(void* options);


		/// <summary>
		/// Set the bitrate of the MPEG audio output stream (LAME style).
		/// </summary>
		/// <param name="bitrate">bitrate in kbps(Default:192)</param>
		/// <returns>0 if successful</returns>
		public int set_brate(int bitrate)
		{
			return twolame_set_brate(options, bitrate);
		}
		[DllImport(dllname)]
		private static extern int twolame_set_brate(void* options, int bitrate);


		/// <summary>
		/// Get the bitrate of the MPEG audio output (LAME style).
		/// </summary>
		/// <returns>output bitrate in kbps</returns>
		public int get_brate()
		{
			return twolame_get_brate(options);
		}
		[DllImport(dllname)]
		private static extern int twolame_get_brate(void* options);

	
		/// <summary>
		/// Set frame padding for the MPEG audio output stream.
		///  i.e. adjust frame sizes to achieve overall target bitrate
		/// </summary>
		/// <param name="padding">padding type(Default:PAD_NO)</param>
		/// <returns>0 if successful</returns>
		public int set_padding(Padding padding)
		{
			return twolame_set_padding(options, padding);
		}
		[DllImport(dllname)]
		private static extern int twolame_set_padding(void* options, Padding padding);

		
		/// <summary>
		/// Get the padding type of the MPEG audio output.
		/// </summary>
		/// <returns>output padding type</returns>
		public Padding get_padding()
		{
			return twolame_get_padding(options);
		}
		[DllImport(dllname)]
		private static extern Padding twolame_get_padding(void* options);


		/// <summary>
		/// Enable/Disable Energy Level Extension.
		/// </summary>
		/// <param name="energylevels">energy level extension state(Default:FALSE)</param>
		/// <returns>0 if successful</returns>
		public int set_energy_levels(bool energylevels)
		{
			return twolame_set_energy_levels(options, energylevels ? 1:0);
		}
		[DllImport(dllname)]
		private static extern int twolame_set_energy_levels(void* options, int energylevels );


		/// <summary>
		/// Get the Energy Level Extension state.
		/// </summary>
		/// <returns>state of the Energy Level Extension (true/false)</returns>
		public bool get_energy_levels()
		{
			return twolame_get_energy_levels(options) == 0 ? false : true;
		}
		[DllImport(dllname)]
		private static extern int twolame_get_energy_levels(void* options);


		/// <summary>
		/// Set number of Ancillary Bits at end of frame.
		/// </summary>
		/// <param name="num">number of bits to reserve(Default:0)</param>
		/// <returns>0 if successful</returns>
		public int set_num_ancillary_bits(int num)
		{
			return twolame_set_num_ancillary_bits(options, num);
		}
		[DllImport(dllname)]
		private static extern int twolame_set_num_ancillary_bits(void* options, int num);


		/// <summary>
		/// Get the number of Ancillary Bits at end of frame.
		/// </summary>
		/// <returns>number of Ancillary Bits at end of frame</returns>
		public int get_num_ancillary_bits()
		{
			return twolame_get_num_ancillary_bits(options);
		}
		[DllImport(dllname)]
		private static extern int twolame_get_num_ancillary_bits(void* options);


		/// <summary>
		/// Set the type of pre-emphasis to be applied to the decoded audio.
		/// </summary>
		/// <param name="emphasis">the type of pre-emphasis(Default:EMPHASIS_N)</param>
		/// <returns>0 if successful</returns>
		public int set_emphasis(Emphasis emphasis)
		{
			return twolame_set_emphasis(options, emphasis);
		}
		[DllImport(dllname)]
		private static extern int twolame_set_emphasis(void* options, Emphasis emphasis);


		/// <summary>
		/// Get the type of pre-emphasis to be applied to the decoded audio.
		/// </summary>
		/// <returns>the type of pre-emphasis</returns>
		public Emphasis get_emphasis()
		{
			return twolame_get_emphasis(options);
		}
		[DllImport(dllname)]
		private static extern Emphasis twolame_get_emphasis(void* options);

	
		/// <summary>
		/// Enable/Disable CRC Error Protection.
		/// </summary>
		/// <param name="err_protection">error protection state(Default:FALSE)</param>
		/// <returns>0 if successful</returns>
		public int set_error_protection(bool err_protection)
		{
			return twolame_set_error_protection(options, err_protection ? 1:0);
		}
		[DllImport(dllname)]
		private static extern int twolame_set_error_protection(void* options, int err_protection);


		/// <summary>
		/// Get the CRC Error Protection state.
		/// </summary>
		/// <returns>state of Error Protection</returns>
		public bool get_error_protection()
		{
			return twolame_get_error_protection(options) == 0 ? false : true;
		}
		[DllImport(dllname)]
		private static extern int twolame_get_error_protection(void* options);

		
		/// <summary>
		/// Set the MPEG Audio Copyright flag.
		///  Indicates that MPEG stream is copyrighted.
		/// </summary>
		/// <param name="copyright">copyright flag state(Default:false)</param>
		/// <returns>0 if successful</returns>
		public int set_copyright(bool copyright)
		{
			return twolame_set_copyright(options, copyright ? 1:0);
		}
		[DllImport(dllname)]
		private static extern int twolame_set_copyright(void* options, int copyright);


		/// <summary>
		/// Get the copright flag state
		/// </summary>
		/// <returns>state of the copyright flag</returns>
		public bool get_copyright()
		{
			return twolame_get_copyright(options) ==0 ? false : true;
		}
		[DllImport(dllname)]
		private static extern int twolame_get_copyright(void* options);

		
		/// <summary>
		/// Set the MPEG Audio Original flag.
		/// </summary>
		/// <param name="original">original flag state(Default:false)</param>
		/// <returns>0 if successful</returns>
		public int set_original(bool original)
		{
			return twolame_set_original(options, original ? 1:0);
		}
		[DllImport(dllname)]
		private static extern int twolame_set_original(void* options, int original);


		/// <summary>
		/// Get the origianl flag state.
		/// </summary>
		/// <returns>state of the original flag</returns>
		public bool get_original()
		{
			return twolame_get_original(options) ==0 ? false : true;
		}
		[DllImport(dllname)]
		private static extern int twolame_get_original(void* options);

		
		/// <summary>
		/// Enable/Disable VBR (Variable Bit Rate) mode.
		/// </summary>
		/// <param name="vbr">VBR state(Default:false)</param>
		/// <returns>0 if successful</returns>
		public int set_VBR(bool vbr)
		{
			return twolame_set_VBR(options, vbr ? 1:0);
		}
		[DllImport(dllname)]
		private static extern int twolame_set_VBR(void* options, int vbr);

		
		/// <summary>
		/// Get the VBR state.
		/// </summary>
		/// <returns>state of VBR</returns>
		public bool get_VBR()
		{
			return twolame_get_VBR(options) ==0 ? false : true;
		}
		[DllImport(dllname)]
		private static extern int twolame_get_VBR(void* options);


		/// <summary>
		/// Set the level/quality of the VBR audio.
		///  The level value can is a measurement of quality - the higher
		///  the level the higher the average bitrate of the resultant file.
		/// </summary>
		/// <param name="level">quality level (-10 to 10) Default:5.0</param>
		/// <returns>0 if successful</returns>
		public int set_VBR_level(float level)
		{
			return twolame_set_VBR_level(options, level);
		}
		[DllImport(dllname)]
		private static extern int twolame_set_VBR_level(void* options, float level);


		/// <summary>
		/// Get the level/quality of the VBR audio.
		/// </summary>
		/// <returns>quality value for VBR</returns>
		public float get_VBR_level()
		{
			return twolame_get_VBR_level(options);
		}
		[DllImport(dllname)]
		private static extern float twolame_get_VBR_level(void* options);


		/// <summary>
		/// Set the adjustment (in dB) applied to the ATH for Psycho models 3 and 4.
		/// </summary>
		/// <param name="level">adjustment level in db(default:0.0)</param>
		/// <returns>0 if successful</returns>
		public int set_ATH_level(float level)
		{
			return twolame_set_ATH_level(options, level);
		}
		[DllImport(dllname)]
		private static extern int twolame_set_ATH_level(void* options, float level);

		
		/// <summary>
		/// Get the adjustment (in dB) applied to the ATH for Psycho models 3 and 4.
		/// </summary>
		/// <returns>adjustment level in db</returns>
		public float get_ATH_level()
		{
			return twolame_get_ATH_level(options);
		}
		[DllImport(dllname)]
		private static extern float twolame_get_ATH_level(void* options);


		/// <summary>
		/// Set the upper bitrate for VBR
		/// </summary>
		/// <param name="bitrate">upper bitrate for VBR(Default: 0 (off))</param>
		/// <returns>non-zero on failure</returns>
		public int set_VBR_max_bitrate_kbps(int bitrate)
		{
			return twolame_set_VBR_max_bitrate_kbps(options, bitrate);
		}
		[DllImport(dllname)]
		private static extern int twolame_set_VBR_max_bitrate_kbps(void* options, int bitrate);


		/// <summary>
		/// Get the upper bitrate for VBR.
		/// </summary>
		/// <returns>the upper bitrate for VBR</returns>
		public int get_VBR_max_bitrate_kbps()
		{
			return twolame_get_VBR_max_bitrate_kbps(options);
		}
		[DllImport(dllname)]
		private static extern int twolame_get_VBR_max_bitrate_kbps(void* options);


		/// <summary>
		/// Enable/Disable the quick mode for psycho model calculation.
		/// </summary>
		/// <param name="quickmode">the state of quick mode (Default:false)</param>
		/// <returns>0 if successful</returns>
		public int set_quick_mode(bool quickmode)
		{
			return twolame_set_quick_mode(options, quickmode ? 1:0);
		}
		[DllImport(dllname)]
		private static extern int twolame_set_quick_mode(void* options, int quickmode);

		
		/// <summary>
		/// Get the state of quick mode.
		/// </summary>
		/// <returns>the state of quick mode</returns>
		public bool get_quick_mode()
		{
			return twolame_get_quick_mode(options) ==0 ? false : true;
		}
		[DllImport(dllname)]
		private static extern int twolame_get_quick_mode(void* options);


		/// <summary>
		/// Set how often the psy model is calculated.
		/// </summary>
		/// <param name="quickcount">umber of frames between calculations(Default:10)</param>
		/// <returns>non-zero on failure</returns>
		public int set_quick_count(int quickcount)
		{
			return twolame_set_quick_count(options, quickcount);
		}
		[DllImport(dllname)]
		private static extern int twolame_set_quick_count(void* options, int quickcount );


		/// <summary>
		/// Get the how often the psy model is calculated.
		/// </summary>
		/// <returns>number of frames between calculations</returns>
		public int get_quick_count()
		{
			return twolame_get_quick_count(options);
		}
		[DllImport(dllname)]
		private static extern int twolame_get_quick_count(void* options);


		/// <summary>
		/// Enable/Disable the Eureka 147 DAB extensions for MP2.
		/// </summary>
		/// <param name="dab">state of DAB extensions (Default:FALSE)</param>
		/// <returns>non-zero on failure</returns>
		public int set_DAB(bool dab)
		{
			return twolame_set_DAB(options, dab ? 1:0);
		}
		[DllImport(dllname)]
		private static extern int twolame_set_DAB(void* options, int dab);


		/// <summary>
		/// Get the state of the DAB extensions
		/// </summary>
		/// <returns>the state of DAB</returns>
		public bool get_DAB()
		{
			return twolame_get_DAB(options) ==0 ? false : true;
		}
		[DllImport(dllname)]
		private static extern int twolame_get_DAB(void* options);

		
		/// <summary>
		/// Set the number of bytes to reserve for DAB XPAD data.
		/// </summary>
		/// <param name="length">number of bytes to reserve(Default:0)</param>
		/// <returns>non-zero on failure</returns>
		public int set_DAB_xpad_length(int length)
		{
			return twolame_set_DAB_xpad_length(options, length);
		}
		[DllImport(dllname)]
		private static extern int twolame_set_DAB_xpad_length(void* options, int length);


		/// <summary>
		/// Get the number of bytes reserved for DAB XPAD data.
		/// </summary>
		/// <returns>number of XPAD bytes</returns>
		public int get_DAB_xpad_length()
		{
			return twolame_get_DAB_xpad_length(options);
		}
		[DllImport(dllname)]
		private static extern int twolame_get_DAB_xpad_length(void* options);


		/// <summary>
		/// Set the CRC error protection length for DAB.
		/// </summary>
		/// <param name="length">length of DAB CRC(Default:2)</param>
		/// <returns>non-zero on failure</returns>
		public int set_DAB_crc_length(int length)
		{
			return twolame_set_DAB_crc_length(options, length);
		}
		[DllImport(dllname)]
		private static extern int twolame_set_DAB_crc_length(void* options, int length);

		
		/// <summary>
		/// Get the CRC error protection length for DAB.
		/// </summary>
		/// <returns>length of DAB CRC</returns>
		public int get_DAB_crc_length()
		{
			return twolame_get_DAB_crc_length(options);
		}
		[DllImport(dllname)]
		private static extern int twolame_get_DAB_crc_length(void* options);
	}
}
